import requests
import re
import os
import csv
from datetime import datetime, timedelta
import json
import time
import logging

# Google Drive Imports
from pydrive.auth import GoogleAuth, ServiceAccountCredentials
from pydrive.drive import GoogleDrive

# Initialize logging
logging.basicConfig(
    filename='data_extraction.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)

console = logging.StreamHandler()
logging.getLogger().addHandler(console)

# Setup for Google Drive
dir_path = os.path.dirname(os.path.realpath(__file__))

scope = ['https://www.googleapis.com/auth/drive']
DRIVE_FOLDER = '1xYymskF1Q2K1z5cGmnMEs410Yt2z7vk_'
CREDENTIALS = os.path.join(dir_path, "Credentials.json")
gauth = GoogleAuth()
gauth.credentials = ServiceAccountCredentials.from_json_keyfile_name(CREDENTIALS, scope)
drive = GoogleDrive(gauth)

def uploadToDrive(file_path):
    # Extract the filename from the file path for uploading
    filename = os.path.basename(file_path)  # Get just the filename
    file = drive.CreateFile({'title': filename, "parents": [{"kind": "drive#fileLink", "id": DRIVE_FOLDER}]})
    file.SetContentFile(file_path)
    file.Upload()
    logging.info(f"{filename} successfully uploaded to Google Drive.")


def clean_old_files(directory, age_in_hours):
    """Removes files from the specified directory that are older than age_in_hours"""
    for filename in os.listdir(directory):
        file_path = os.path.join(directory, filename)
        if os.path.isfile(file_path):
            # Get the file's last modified time and the current time
            file_mod_time = datetime.fromtimestamp(os.path.getmtime(file_path))
            current_time = datetime.now()
            # If the file is older than the specified age, delete it
            if current_time - file_mod_time > timedelta(hours=age_in_hours):
                os.remove(file_path)
                logging.info(f"Removed old file: {filename}")


# Function to extract data and save to CSV
def getDataAndSaveToCSV(service_name, api_key):
    logging.info(f"Starting data retrieval for {service_name}")

    # Set up the endpoint and headers
    url = 'https://api.ebay.com/ws/api.dll'
    headers = {
        'Content-Type': 'text/xml',
        'X-EBAY-API-CALL-NAME': 'GetSellerList',
        'X-EBAY-API-APP-NAME': 'YOUR-APP-ID',
        'X-EBAY-API-DEV-NAME': 'YOUR-DEV-ID',
        'X-EBAY-API-CERT-NAME': 'YOUR-CERT-ID',
        'X-EBAY-API-SITEID': '3',
        'X-EBAY-API-COMPATIBILITY-LEVEL': '967'
    }

    # Adjust the entries per page as needed
    entries_per_page = 200
    total_entries_retrieved = 0
    extracted_data = []
    page_number = 1

    # Calculate the dates
    end_date = datetime.utcnow()
    start_date = end_date - timedelta(days=119)

    while True:
        # Create the XML request body
        body = f"""
        <?xml version="1.0" encoding="utf-8"?>
        <GetSellerListRequest xmlns="urn:ebay:apis:eBLBaseComponents">
          <RequesterCredentials>
            <eBayAuthToken>{api_key}</eBayAuthToken>
          </RequesterCredentials>
          <UserID>{service_name}</UserID>
          <DetailLevel>ReturnAll</DetailLevel>
          <IncludeVariations>true</IncludeVariations>
          <StartTimeFrom>{start_date.strftime('%Y-%m-%dT%H:%M:%S.000Z')}</StartTimeFrom>
          <StartTimeTo>{end_date.strftime('%Y-%m-%dT%H:%M:%S.000Z')}</StartTimeTo>
         <Pagination>
            <EntriesPerPage>{entries_per_page}</EntriesPerPage>
            <PageNumber>{page_number}</PageNumber>
          </Pagination>
        </GetSellerListRequest>
        """

        # Make the request
        response = requests.post(url, headers=headers, data=body)
        logging.info(f"Fetched page {page_number}")

        # Check rate limit and add delay if necessary
        if total_entries_retrieved > 0 and total_entries_retrieved % 300 == 0:
            logging.warning("Approaching rate limit - pausing for 15 seconds")
            time.sleep(15)

        # Process the response
        if response.status_code == 200:
            xml_data = response.text
            items = re.findall(r'<Item>(.*?)</Item>', xml_data, re.DOTALL)
            if not items:
                logging.info("No more items to retrieve.")
                break

            markers = {
                "Name": r"<Title>(.*?)</Title>",
                "Item URL": r"<ViewItemURL>(.*?)</ViewItemURL>",
                "Price": r"<SellingStatus>.*?<CurrentPrice.*?>(.*?)</CurrentPrice>.*?</SellingStatus>",
                "Stock Quantity": r"<Quantity>(.*?)</Quantity>",
                "Delivery Template": r"<ShippingProfileName>(.*?)</ShippingProfileName>",
                "SKU": r"<SKU>(.*?)</SKU>",
                "Item Number": r"<ItemID>(.*?)</ItemID>"
            }

            for item in items:
                listing_status = re.search(r'<ListingStatus>(.*?)</ListingStatus>', item, re.DOTALL)
                if listing_status and listing_status.group(1) != 'Active':
                    continue  # Skip if the listing is not active

                data = {}
                for key, marker in markers.items():
                    match = re.search(marker, item, re.DOTALL)
                    data[key] = match.group(1) if match else 'N/A'

                data["Is Variation"] = 'Yes' if re.search(r'<Variations>', item, re.DOTALL) else 'No'
                data["Time Handled"] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                extracted_data.append(data)

            total_entries_retrieved += len(items)
            logging.info(f"Total entries retrieved so far: {total_entries_retrieved}")
            page_number += 1
        else:
            logging.error(f"Failed to retrieve data for {service_name} on page {page_number}: {response.status_code}")
            break

    # Moved directory check and file writing outside the while loop

    # Define the directory path for data folder
    data_dir = os.path.join(dir_path, 'data')
    # Check if the directory exists, if not, create it
    if not os.path.exists(data_dir):
        os.makedirs(data_dir)

    # Create and save data to a CSV file
    filename = f"{service_name} Data {datetime.now().strftime('%d.%m.%Y %H.%M')}.csv"
    file_path = os.path.join(data_dir, filename)

    with open(file_path, mode='w', newline='', encoding='utf-8') as file:
        fieldnames = [
            "Name", "Item URL", "Price", "Stock Quantity",
            "Delivery Template", "SKU", "Item Number", "Is Variation", "Time Handled"
        ]
        writer = csv.DictWriter(file, fieldnames=fieldnames)
        writer.writeheader()
        writer.writerows(extracted_data)

    logging.info(f"Data saved for {service_name} to {filename}, total items: {len(extracted_data)}")

    # Upload file to Google Drive
    uploadToDrive(file_path)  # Pass the full file path


# Load eBay services config
with open('eBay Services Config.json') as f:
    config = json.load(f)

# Loop through each service and get data
for i in range(1, (len(config) // 2) + 1):
    service_name_key = f"Service {i}"
    api_key_key = f"API Key {i}"
    getDataAndSaveToCSV(config[service_name_key], config[api_key_key])

logging.info("All data retrieval completed.")

# Usage of the function
data_dir = os.path.join(dir_path, 'data')
clean_old_files(data_dir, 48)
